@@ -21,8 +21,9 @@ use strict;
 use vars qw($VERSION %samsungLensTypes);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
+use Image::ExifTool::JSON;
 
-$VERSION = '1.58';
+$VERSION = '1.59';
 
 sub WriteSTMN($$$);
 sub ProcessINFO($$$);
@@ -1277,7 +1278,15 @@ my %formatMinMax = (
    # 0x0b51-name - seen 'Intelligent_PhotoEditor_Data' #forum16086
    # 0x0b60-name - seen 'UltraWide_PhotoEditor_Data' #forum16086
    # 0x0b90-name - seen 'Document_Scan_Info' #forum16086
-   # 0x0ba1-name - seen 'Original_Path_Hash_Key', 'PhotoEditor_Re_Edit_Data', 'deco_doodle_bitmap', 'deco_sticker_bitmap', 'deco_text_bitmap'
+   # 0x0ba1-name - seen 'Original_Path_Hash_Key', 'PhotoEditor_Re_Edit_Data', 'deco_doodle_bitmap', 'deco_sticker_bitmap', 'deco_text_bitmap','PhotoEditor_Re_Edit_Data'
+    '0x0ba1' => [{
+        Name => 'ReEditData',
+        Condition => '$$self{SamsungTagName} eq "PhotoEditor_Re_Edit_Data"',
+        SubDirectory => { TagTable => 'Image::ExifTool::Samsung::ReEditData' },
+    },{
+        Name => 'OriginalPathHashKey',
+        Condition => '$$self{SamsungTagName} eq "Original_Path_Hash_Key"',
+    }],
    # 0x0ba2-name - seen 'Copy_Available_Edit_Info' #forum16086
    # 0x0bc0-name - seen 'Single_Relighting_Bokeh_Info' #forum16086
    # 0x0bd0-name - seen 'Dual_Relighting_Bokeh_Info' #forum16086
@@ -1300,7 +1309,7 @@ my %formatMinMax = (
     '0x0d91' => { #forum16086/16242
         Name => 'PEg_Info',
         Description => 'PEg Info',
-        SubDirectory => { TagTable => 'Image::ExifTool::JSON::Main' },
+        SubDirectory => { TagTable => 'Image::ExifTool::Samsung::PEgInfo' },
     },
    # 0x0da1-name - seen 'Captured_App_Info' #forum16086
    # 0xa050-name - seen 'Jpeg360_2D_Info' (Samsung Gear 360)
@@ -1380,6 +1389,113 @@ my %formatMinMax = (
     perfMode            => { },
 );
 
+%Image::ExifTool::Samsung::ReEditData = (
+    GROUPS => { 0 => 'JSON', 2 => 'Image' },
+    PROCESS_PROC => \&Image::ExifTool::JSON::ProcessJSON,
+    VARS => { LONG_TAGS => 2 },
+    originalPath        => { },
+    representativeFrameLoc => { },
+    startMotionVideo    => { },
+    endMotionVideo      => { },
+    isMotionVideoMute   => { },
+    isTrimMotionVideo   => { },
+    clipInfoValue       => { SubDirectory => { TagTable => 'Image::ExifTool::Samsung::ClipInfo' } },
+    toneValue           => { SubDirectory => { TagTable => 'Image::ExifTool::Samsung::ToneInfo' } },
+    effectValue         => { SubDirectory => { TagTable => 'Image::ExifTool::Samsung::EffectInfo' } },
+    portraitEffectValue => { SubDirectory => { TagTable => 'Image::ExifTool::Samsung::PortraitEffect' }  },
+    isBlending          => { },
+    isNotReEdit         => { },
+    sepVersion          => { Name => 'SEPVersion' },
+    ndeVersion          => { Name => 'NDEVersion' },
+    reSize              => { },
+    isScaleAI           => { },
+    rotation            => { },
+    adjustmentValue     => { },
+    isApplyShapeCorrection => { },
+    isNewReEditOnly     => { },
+    isDecoReEditOnly    => { },
+    isAIFilterReEditOnly=> { },
+);
+
+%Image::ExifTool::Samsung::ClipInfo = (
+    GROUPS => { 0 => 'JSON', 2 => 'Image' },
+    PROCESS_PROC => \&Image::ExifTool::JSON::ProcessJSON,
+    mCenterX        => { Name => 'ClipCenterX' },
+    mCenterY        => { Name => 'ClipCenterY' },
+    mWidth          => { Name => 'ClipWidth' },
+    mHeight         => { Name => 'ClipHeight' },
+    mRotation       => { Name => 'ClipRotation' },
+    mRotate         => { Name => 'ClipRotate' },
+    mHFlip          => { Name => 'ClipHFlip' },
+    mVFlip          => { Name => 'ClipVFlip' },
+    mRotationEffect => { Name => 'ClipRotationEffect' },
+    mRotateEffect   => { Name => 'ClipRotateEffect' },
+    mHFlipEffect    => { Name => 'ClipHFlipEffect' },
+    mVFlipEffect    => { Name => 'ClipVFlipEffect' },
+    mHozPerspective => { Name => 'ClipHozPerspective' },
+    mVerPerspective => { Name => 'ClipVerPerspective' },
+);
+
+%Image::ExifTool::Samsung::ToneInfo = (
+    GROUPS => { 0 => 'JSON', 2 => 'Image' },
+    PROCESS_PROC => \&Image::ExifTool::JSON::ProcessJSON,
+    brightness      => { },
+    exposure        => { },
+    contrast        => { },
+    saturation      => { },
+    hue             => { },
+    wbMode          => { Name => 'WBMode' },
+    wbTemperature   => { Name => 'WBTemperature' },
+    tint            => { },
+    shadow          => { },
+    highlight       => { },
+    lightbalance    => { },
+    sharpness       => { },
+    definition      => { },
+    isBrightnessIPE => { },
+    isExposureIPE   => { },
+    isContrastIPE   => { },
+    isSaturationIPE => { },
+);
+
+%Image::ExifTool::Samsung::EffectInfo = (
+    GROUPS => { 0 => 'JSON', 2 => 'Image' },
+    PROCESS_PROC => \&Image::ExifTool::JSON::ProcessJSON,
+    filterIndication=> { },
+    alphaValue      => { },
+    filterType      => { },
+);
+
+%Image::ExifTool::Samsung::PortraitEffect = (
+    GROUPS => { 0 => 'JSON', 2 => 'Image' },
+    PROCESS_PROC => \&Image::ExifTool::JSON::ProcessJSON,
+    VARS => { LONG_TAGS => 1 },
+    effectId        => { Name => 'PortraitEffectID' },
+    effectLevel     => { Name => 'PortraitEffectLevel' },
+    exifRotation    => { Name => 'PortraitExifRotation' },
+    lightLevel      => { Name => 'PortraitLightLevel' },
+    touchX          => { Name => 'PortraitTouchX' },
+    touchY          => { Name => 'PortraitTouchY' },
+    refocusX        => { Name => 'PortraitRefocusX' },
+    refocusY        => { Name => 'PortraitRefocusY' },
+    effectIdOriginal    => { Name => 'PortraitEffectIDOriginal' },
+    effectLevelOriginal => { Name => 'EffectLevelOriginal' },
+    lightLevelOriginal  => { Name => 'LightLevelOriginal' },
+    touchXOriginal  => { },
+    touchYOriginal  => { },
+    refocusXOriginal=> { },
+    refocusYOriginal=> { },
+    waterMarkRemoved=> { Name => 'WaterMarkRemoved' },
+    waterMarkRemovedOriginal => { Name => 'WaterMarkRemovedOriginal' },
+);
+
+%Image::ExifTool::Samsung::PEgInfo = (
+    GROUPS => { 0 => 'JSON', 2 => 'Image' },
+    PROCESS_PROC => \&Image::ExifTool::JSON::ProcessJSON,
+    genImageVersion => { },
+    connectorType   => { },
+);
+
 # Samsung composite tags
 %Image::ExifTool::Samsung::Composite = (
     GROUPS => { 2 => 'Image' },
@@ -1625,7 +1741,14 @@ SamBlock:
             }
             next;
         }
-        last unless $buff =~ /^SEFH/ and $len >= 12;   # validate SEFH header
+        # validate SEFH header
+        unless ($buff =~ /^SEFH/ and $len >= 12) {
+            # tolerate extra junk written by Samsung Gallery
+            last unless $buff =~ /\0\0SEFT/g;
+            $et->Warn('Trailer likely corrupted by Samsung Gallery');
+            $blockEnd += pos($buff);
+            next;
+        }
         my $dirPos = $raf->Tell() - $len;
         # my $ver = Get32u(\$buff, 0x04);  # version (=101)
         my $count = Get32u(\$buff, 0x08);
